@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_vde.cc,v 1.8.4.1 2005/07/07 07:07:31 vruppert Exp $
+// $Id: eth_vde.cc,v 1.11 2005/12/10 18:37:35 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2003  Renzo Davoli
@@ -26,7 +26,9 @@
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
  
+#define NO_DEVICE_INCLUDES
 #include "iodev.h"
+
 #if BX_NETWORKING && defined(HAVE_VDE)
 
 #include "eth.h"
@@ -228,11 +230,6 @@ void bx_vde_pktmover_c::rx_timer ()
 
   rxbuf=buf;
 
-  // hack: TUN/TAP device likes to create an ethernet header which has
-  // the same source and destination address FE:FD:00:00:00:00.
-  // Change the dest address to FE:FD:00:00:00:01.
-  rxbuf[5] = 1;
-
   if (nbytes>0)
     BX_INFO (("vde read returned %d bytes", nbytes));
   if (nbytes<0) {
@@ -272,8 +269,8 @@ void bx_vde_pktmover_c::rx_timer ()
 #define REQ_NEW_CONTROL 0
 
 struct request_v3 {
-	Bit32u_t magic;
-	Bit32u_t version;
+	Bit32u magic;
+	Bit32u version;
 	//enum request_type type;
 	int type;
 	struct sockaddr_un sock;
