@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit.h,v 1.10 2002/10/25 11:44:40 bdenney Exp $
+// $Id: pit.h,v 1.13 2004/12/13 19:10:38 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -71,9 +71,6 @@ public:
   BX_PIT_SMF void reset( unsigned type);
   BX_PIT_SMF bx_bool periodic( Bit32u   usec_delta );
 
-  BX_PIT_SMF int SaveState( class state_file *fd );
-  BX_PIT_SMF int LoadState( class state_file *fd );
-
 private:
 
   static Bit32u read_handler(void *this_ptr, Bit32u address, unsigned io_len);
