@@ -1,20 +1,18 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pit82c54.h,v 1.12 2003/03/02 23:59:11 cbothamy Exp $
+// $Id: pit82c54.h,v 1.14 2006/01/08 20:39:08 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 /*
  * Emulator of an Intel 8254/82C54 Programmable Interval Timer.
  * Greg Alexander <yakovlev@usa.com>
- *
- * This code is not yet linked into Bochs, but has been included so
- * that you can experiment with it.  (bbd)
  */
 
 #ifndef _PIT_82C54_H_
 #define _PIT_82C54_H_ 1
 
-#include "bochs.h"
+//#include "bochs.h"
 
+typedef void (*out_handler_t)(bx_bool value);
 
 class pit_82C54 : public logfunctions {
 
@@ -51,8 +49,8 @@ private:
 
   struct counter_type {
     //Chip IOs;
-    bool GATE; //GATE Input value at end of cycle
-    bool OUTpin; //OUT output this cycle
+    bx_bool GATE; //GATE Input value at end of cycle
+    bx_bool OUTpin; //OUT output this cycle
 
     //Architected state;
     Bit32u count; //Counter value this cycle
@@ -63,25 +61,26 @@ private:
     //Status Register data;
     Bit8u rw_mode; //2-bit R/W mode from command word register.
     Bit8u mode; //3-bit mode from command word register.
-    bool bcd_mode; //1-bit BCD vs. Binary setting.
-    bool null_count; //Null count bit of status register.
+    bx_bool bcd_mode; //1-bit BCD vs. Binary setting.
+    bx_bool null_count; //Null count bit of status register.
 
     //Latch status data;
-    bool count_LSB_latched;
-    bool count_MSB_latched;
-    bool status_latched;
+    bx_bool count_LSB_latched;
+    bx_bool count_MSB_latched;
+    bx_bool status_latched;
 
     //Miscelaneous State;
     Bit32u count_binary; //Value of the count in binary.
-    bool triggerGATE; //Whether we saw GATE rise this cycle.
+    bx_bool triggerGATE; //Whether we saw GATE rise this cycle.
     rw_status write_state; //Read state this cycle
     rw_status read_state; //Read state this cycle
-    bool count_written; //Whether a count written since programmed
-    bool first_pass; //Whether or not this is the first loaded count.
-    bool state_bit_1; //Miscelaneous state bits.
-    bool state_bit_2;
+    bx_bool count_written; //Whether a count written since programmed
+    bx_bool first_pass; //Whether or not this is the first loaded count.
+    bx_bool state_bit_1; //Miscelaneous state bits.
+    bx_bool state_bit_2;
     Bit32u next_change_time; //Next time something besides count changes.
                              //0 means never.
+    out_handler_t out_handler; // OUT pin callback (for IRQ0)
   };
 
   counter_type counter[3];
@@ -92,7 +91,7 @@ private:
 
   void latch_counter(counter_type & thisctr);
 
-  void set_OUT (counter_type & thisctr, bool data);
+  void set_OUT (counter_type & thisctr, bx_bool data);
 
   void set_count (counter_type & thisctr, Bit32u data) BX_CPP_AttrRegparmN(2);
 
@@ -119,13 +118,15 @@ public:
   Bit8u read(Bit8u address);
   void write(Bit8u address, Bit8u data);
 
-  void set_GATE(Bit8u cnum, bool data);
-  bool read_GATE(Bit8u cnum);
+  void set_GATE(Bit8u cnum, bx_bool data);
+  bx_bool read_GATE(Bit8u cnum);
 
-  bool read_OUT(Bit8u cnum);
+  bx_bool read_OUT(Bit8u cnum);
+  void set_OUT_handler(Bit8u cnum, out_handler_t outh);
 
   Bit32u get_clock_event_time(Bit8u cnum);
   Bit32u get_next_event_time(void);
+  Bit16u get_inlatch(int countnum);
 
   void print_cnum(Bit8u cnum);
 
